/*
 * WiFly Tutorial Example
 * Copyright (c) 2010 SparkFun Electronics.  All right reserved.
 * Written by Chris Taylor
 *
 * This code was written to demonstrate the WiFly Tutorial from SparkFun Electronics
 * 
 * This code will associate with the SparkFun WiFi network, serve a simple page, and 
 * "say" typed words using the Voice Box Shield
 *
 * http://www.sparkfun.com
 */

#include <string.h>
#include <SoftwareSerial.h>
#include <avr/pgmspace.h>

// SC16IS750 Register Definitions
#define THR        0x00 << 3
#define RHR        0x00 << 3
#define IER        0x01 << 3
#define FCR        0x02 << 3
#define IIR        0x02 << 3
#define LCR        0x03 << 3
#define MCR        0x04 << 3
#define LSR        0x05 << 3
#define MSR        0x06 << 3
#define SPR        0x07 << 3
#define TXFIFO     0x08 << 3
#define RXFIFO     0x09 << 3
#define DLAB       0x80 << 3
#define IODIR      0x0A << 3
#define IOSTATE    0x0B << 3
#define IOINTMSK   0x0C << 3
#define IOCTRL     0x0E << 3
#define EFCR       0x0F << 3

#define DLL        0x00 << 3
#define DLM        0x01 << 3
#define EFR        0x02 << 3
#define XON1       0x04 << 3  
#define XON2       0x05 << 3
#define XOFF1      0x06 << 3
#define XOFF2      0x07 << 3

// SPI port definitions
#define CS         10
#define MOSI       11
#define MISO       12
#define SCK        13

// Voice Box Shield definitions
#define RES  3
#define SPK  4
#define txPin  2
SoftwareSerial speakjet = SoftwareSerial(0, txPin);

#define WORD_LIST_LENGTH 100

#define ASSOCIATE_TIMEOUT 5000

char is_get = 1;

// "Associated"
char message[] = {
  132,187,187,137,8,189,8,128,154,191,129,176,0,255};

// SpeakJet numbers
char zero[] = {
  167,7,128,7,149,164,0,255};
char one[] = {
  147,14,135,8,141,0,255};
char two[] = {
  8,191,162,31,0,255};
char three[] = {
  8,190,148,8,128,0,255};
char four[] = {
  186,7,137,153,0,0,255};
char five[] = {
  186,157,166,0,255};
char six[] = {
  8,187,129,14,194,7,187,0,255};
char seven[] = {
  8,187,7,131,166,131,141,0,255};
char eight[] = {
  154,4,191,0,255};
char nine[] = {
  141,14,157,141,0,255};

char *numbers[] = { 
  zero, one, two, three, four, five, six, seven, eight, nine };

char ping[] = {
  220,0,255};
char dot[] = {
  175,8,135,191,0,255};


// WORD LIST 
// Phoneme strings for SpeakJet 
// Placed in program memory to save RAM
prog_char sj_the[] PROGMEM = {
  8,169,8,128,0,0,255};
prog_char sj_of[] PROGMEM = {
  8,134,166,0,0,255};
prog_char sj_and1[] PROGMEM = {
  8,132,8,141,177,0,0,255};
prog_char sj_a1[] PROGMEM = {
  154,128,0,0,255};
prog_char sj_to[] PROGMEM = {
  8,191,162,0,0,255};
prog_char sj_in[] PROGMEM = {
  8,129,8,141,0,0,255};
prog_char sj_is[] PROGMEM = {
  8,129,167,0,0,255};
prog_char sj_you[] PROGMEM = {
  8,160,0,0,255};
prog_char sj_that[] PROGMEM = {
  169,8,132,8,191,0,0,255};
prog_char sj_it[] PROGMEM = {
  8,129,8,191,0,0,255};
prog_char sj_he[] PROGMEM = {
  183,128,0,0,255};
prog_char sj_for1[] PROGMEM = {
  186,153,0,0,255};
prog_char sj_was[] PROGMEM = {
  147,134,167,0,0,255};
prog_char sj_on[] PROGMEM = {
  135,135,142,0,0,255};
prog_char sj_are[] PROGMEM = {
  152,0,0,255};
prog_char sj_as[] PROGMEM = {
  132,8,167,0,0,255};
prog_char sj_with[] PROGMEM = {
  147,129,8,8,190,0,0,255};
prog_char sj_his[] PROGMEM = {
  183,129,167,0,0,255};
prog_char sj_they[] PROGMEM = {
  8,169,154,0,0,255};
prog_char sj_at[] PROGMEM = {
  132,8,191,0,0,255};
prog_char sj_be[] PROGMEM = {
  170,128,128,0,0,255};
prog_char sj_this1[] PROGMEM = {
  8,169,8,129,187,0,0,255};
prog_char sj_from[] PROGMEM = {
  186,148,134,8,140,0,0,255};
prog_char sj_i1[] PROGMEM = {
  157,0,0,255};
prog_char sj_have[] PROGMEM = {
  183,8,132,166,0,0,255};
prog_char sj_or1[] PROGMEM = {
  153,0,0,255};
prog_char sj_by[] PROGMEM = {
  171,157,0,0,255};
prog_char sj_one[] PROGMEM = {
  147,14,135,8,141,0,0,255};
prog_char sj_not1[] PROGMEM = {
  141,135,191,0,0,255};
prog_char sj_what[] PROGMEM = {
  185,8,135,191,0,0,255};
prog_char sj_all[] PROGMEM = {
  8,136,8,146,0,0,255};
prog_char sj_when[] PROGMEM = {
  185,8,131,8,141,0,0,255};
prog_char sj_we[] PROGMEM = {
  147,8,128,0,0,255};
prog_char sj_there[] PROGMEM = {
  8,169,150,0,0,255};
prog_char sj_can[] PROGMEM = {
  194,8,132,141,0,0,255};
prog_char sj_an[] PROGMEM = {
  132,132,141,0,0,255};
prog_char sj_your[] PROGMEM = {
  128,153,0,0,255};
prog_char sj_which[] PROGMEM = {
  185,129,8,182,0,0,255};
prog_char sj_their[] PROGMEM = {
  8,169,150,0,0,255};
prog_char sj_if1[] PROGMEM = {
  129,186,186,0,0,255};
prog_char sj_do1[] PROGMEM = {
  174,162,0,0,255};
prog_char sj_will[] PROGMEM = {
  147,129,8,145,0,0,255};
prog_char sj_each[] PROGMEM = {
  128,128,4,182,0,0,255};
prog_char sj_about[] PROGMEM = {
  134,173,163,191,0,0,255};
prog_char sj_how[] PROGMEM = {
  184,8,163,0,0,255};
prog_char sj_up[] PROGMEM = {
  8,134,199,0,0,255};
prog_char sj_out[] PROGMEM = {
  163,191,0,0,255};
prog_char sj_them[] PROGMEM = {
  8,169,131,8,140,0,0,255};
prog_char sj_then[] PROGMEM = {
  8,169,131,8,142,0,0,255};
prog_char sj_she[] PROGMEM = {
  8,189,8,128,0,0,255};
prog_char sj_many[] PROGMEM = {
  140,131,141,128,0,0,255};
prog_char sj_some[] PROGMEM = {
  8,187,134,140,0,0,255};
prog_char sj_so[] PROGMEM = {
  8,188,7,164,7,147,0,0,255};
prog_char sj_these[] PROGMEM = {
  8,169,8,128,7,167,0,0,255};
prog_char sj_would[] PROGMEM = {
  147,8,138,177,0,0,255};
prog_char sj_other[] PROGMEM = {
  134,190,151,0,0,255};
prog_char sj_into[] PROGMEM = {
  129,141,191,162,0,0,255};
prog_char sj_more[] PROGMEM = {
  140,7,137,153,0,0,255};
prog_char sj_her[] PROGMEM = {
  183,151,0,0,255};
prog_char sj_two[] PROGMEM = {
  8,191,162,0,0,255};
prog_char sj_like[] PROGMEM = {
  145,7,135,7,155,196,0,0,255};
prog_char sj_him[] PROGMEM = {
  183,8,129,140,0,0,255};
prog_char sj_see[] PROGMEM = {
  187,187,128,128,0,0,255};
prog_char sj_time[] PROGMEM = {
  8,191,157,8,140,0,0,255};
prog_char sj_could[] PROGMEM = {
  195,138,138,177,0,0,255};
prog_char sj_no[] PROGMEM = {
  142,164,0,0,255};
prog_char sj_make[] PROGMEM = {
  140,154,196,0,0,255};
prog_char sj_than[] PROGMEM = {
  169,8,132,8,142,0,0,255};
prog_char sj_first[] PROGMEM = {
  186,151,187,191,0,0,255};
prog_char sj_now[] PROGMEM = {
  142,163,0,0,255};
prog_char sj_people[] PROGMEM = {
  198,7,128,198,8,145,0,0,255};
prog_char sj_my[] PROGMEM = {
  140,155,0,0,255};
prog_char sj_over[] PROGMEM = {
  8,137,7,166,151,0,0,255};
prog_char sj_down[] PROGMEM = {
  175,163,141,0,0,255};
prog_char sj_only[] PROGMEM = {
  137,142,145,128,0,0,255};
prog_char sj_find[] PROGMEM = {
  186,155,141,177,0,0,255};
prog_char sj_use[] PROGMEM = {
  160,167,0,0,255};
prog_char sj_may[] PROGMEM = {
  140,154,0,0,255};
prog_char sj_water[] PROGMEM = {
  147,135,191,7,151,0,0,255};
prog_char sj_long1[] PROGMEM = {
  146,135,8,144,0,0,255};
prog_char sj_little[] PROGMEM = {
  145,129,191,131,145,0,0,255};
prog_char sj_feed[] PROGMEM = {
  186,128,128,176,0,0,255};
prog_char sj_very[] PROGMEM = {
  166,150,7,128,0,0,255};
prog_char sj_after[] PROGMEM = {
  132,186,191,7,151,0,0,255};
prog_char sj_me[] PROGMEM = {
  140,128,128,0,0,255};
prog_char sj_words[] PROGMEM = {
  147,151,176,7,167,0,0,255};
prog_char sj_just[] PROGMEM = {
  165,133,187,191,0,0,255};
prog_char sj_cat[] PROGMEM = {
  194,132,191,0,0,255};
prog_char sj_where[] PROGMEM = {
  147,150,0,0,255};
prog_char sj_most[] PROGMEM = {
  140,8,137,187,191,0,0,255};
prog_char sj_know[] PROGMEM = {
  142,137,164,0,0,255};
prog_char sj_dead[] PROGMEM = {
  174,131,174,0,0,255};
prog_char sj_get[] PROGMEM = {
  8,178,8,131,191,0,0,255};
prog_char sj_through[] PROGMEM = {
  8,190,148,8,139,0,0,255};
prog_char sj_back[] PROGMEM = {
  170,8,132,4,196,0,0,255};
prog_char sj_much[] PROGMEM = {
  140,134,182,0,0,255};
prog_char sj_go[] PROGMEM = {
  8,179,8,164,0,0,255};
prog_char sj_good[] PROGMEM = {
  8,179,138,138,177,0,0,255};
prog_char sj_new1[] PROGMEM = {
  141,160,0,0,255};
prog_char sj_write[] PROGMEM = {
  148,155,191,0,0,255};

PROGMEM const char *SpeakJetDictionary[] = {
  sj_the,sj_of,sj_and1,sj_a1,sj_to,sj_in,sj_is,sj_you,sj_that,sj_it,sj_he,sj_for1,sj_was,sj_on,sj_are,sj_as,
  sj_with,sj_his,sj_they,sj_at,sj_be,sj_this1,sj_from,sj_i1,sj_have,sj_or1,sj_by,sj_one,sj_not1,sj_what,sj_all,
  sj_when,sj_we,sj_there,sj_can,sj_an,sj_your,sj_which,sj_their,sj_if1,sj_do1,sj_will,sj_each,sj_about,sj_how,
  sj_up,sj_out,sj_them,sj_then,sj_she,sj_many,sj_some,sj_so,sj_these,sj_would,sj_other,sj_into,sj_more,sj_her,
  sj_two,sj_like,sj_him,sj_see,sj_time,sj_could,sj_no,sj_make,sj_than,sj_first,sj_now,sj_people,sj_my,sj_over,
  sj_down,sj_only,sj_find,sj_use,sj_may,sj_water,sj_long1,sj_little,sj_feed,sj_very,sj_after,sj_me,sj_words,
  sj_just,sj_cat,sj_where,sj_most,sj_know,sj_dead,sj_get,sj_through,sj_back,sj_much,sj_go,sj_good,sj_new1,sj_write

};

// SpeakJet lookup table
// Local searchable table to find indices for phoneme strings
char *SpeakJetLookup[] = {
  "the","of","and","a","to","in","is","you","that","it","he","for","was","on","are","as","with","his","they","at","be","this",
  "from","i","have","or","by","one","not","what","all","when","we","there","can","an","your","which","their","if","do","will",
  "each","about","how","up","out","them","then","she","many","some","so","these","would","other","into","more","her","two","like",
  "him","see","time","could","no","make","than","first","now","people","my","over","down","only","find","use","may","water","long",
  "little","feed","very","after","me","words","just","cat","where","most","know","dead","get","through","back","much","go","good",
  "new","write"

};

// Flags
char incoming_data; 
char SPI_init_flag = 0;
char polling = 0;

char TX_Fifo_Address = THR; 

int i = 0;
int j = 0;
int k = 0;
char clr = 0;

struct SPI_UART_cfg
// Defines register values for SC16IS750
{
  char DivL,DivM,DataFormat,Flow;
};

struct SPI_UART_cfg SPI_Uart_config = {
  0x50,0x00,0x03,0x10};

char auth_level[] = "3";
char auth_phrase[] = "ExamplePassPhrase";
char port_listen[] = "80";
char ssid[] = "SparkFunWireless";

char ip_address[30]; // Stores IP address for SpeakJet

void setup()
{
  // SPI Init
  pinMode(MOSI, OUTPUT);
  pinMode(MISO, INPUT);
  pinMode(SCK,OUTPUT);
  pinMode(CS,OUTPUT);
  digitalWrite(CS,HIGH); //disable device 

  // SpeakJet Init
  pinMode(txPin, OUTPUT);
  pinMode(SPK, INPUT);
  pinMode(RES, OUTPUT);

  // SPI Start
  SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR1)|(1<<SPR0);
  clr=SPSR;
  clr=SPDR;
  delay(10); 
  Serial.begin(9600);
  speakjet.begin(9600); 

  // Reset the SpeakJet
  digitalWrite(RES, LOW);
  delay(100);
  digitalWrite(RES, HIGH);

  Serial.println("\n\r\n\rWiFly Shield Terminal Routine");

  // Initialize and test the SPI-Uart bridge
  if(SPI_Uart_Init()){ 
    Serial.println("Bridge initialized successfully!"); 
  }
  else{ 
    Serial.println("Could not initialize bridge, locking up.\n\r"); 
    while(1); 
  }

  // Autoconnect to local WiFi using parameters from the global vars
  autoconnect();
}

void loop()
{
  // Exit command mode if we haven't already
  SPI_Uart_println("exit");
  delay(500);
  Flush_RX();
  while(1)
  {
    while(!Have_Client()); // Wait on client
    Serial.println("\n\rConnection opened.");

    // Send HTML
    HTML_print("<html>");
    HTML_print("<title>SparkFun SpeakJet Server</title>");
    HTML_print("<h1><center>");
    HTML_print("<img src=\"http://static.sparkfun.com/images/framework/logo.gif\">");
    HTML_print("</center></h1>");
    HTML_print("Type some shizz up:");
    HTML_print("<form name=\"input\" action=\"\" method=\"post\">");
    HTML_print("<input type=\"text\" name=\"%\" size=\"100\">");
    HTML_print("<input type=\"submit\" value=\"Submit\">");
    HTML_print("</form>");
    // Send word list
    HTML_print("<h1>Word List:</h1><p>");
    for(int wordn = 0; wordn < WORD_LIST_LENGTH; wordn++)
    {
      HTML_print(SpeakJetLookup[wordn]);
      HTML_print("  ");
    }
    HTML_print("</html>");
    delay(500);

    // Enter command mode and close connection
    SPI_Uart_print("$$$");
    delay(500);
    SPI_Uart_println("close");
    delay(1000);
    SPI_Uart_println("exit");
    delay(500);
    Serial.println("Connection closed.\n\r");
    Flush_RX();
  } 
}

void select(void) // Select SC16IS750
{
  digitalWrite(CS,LOW);
}

void deselect(void) // Deselect SC16IS750
{
  digitalWrite(CS,HIGH);
}

char Have_Client(void)
{
  if(SPI_Uart_ReadByte(LSR) & 0x01){  // Wait for characters from a connection
    Serial.println("Client request..."); 
    is_get = 1;
    Parse_Request(); // Check if request is a POST
    Flush_RX();
    return 1; 
  }   
  else{ 
    return 0; 
  } 
}

char SPI_Uart_Init(void)
// Initialize SC16IS750
{
  char data = 0;

  SPI_Uart_WriteByte(LCR,0x80); // 0x80 to program baudrate
  SPI_Uart_WriteByte(DLL,SPI_Uart_config.DivL); //0x50 = 9600 with Xtal = 12.288MHz
  SPI_Uart_WriteByte(DLM,SPI_Uart_config.DivM); 

  SPI_Uart_WriteByte(LCR, 0xBF); // access EFR register
  SPI_Uart_WriteByte(EFR, SPI_Uart_config.Flow); // enable enhanced registers
  SPI_Uart_WriteByte(LCR, SPI_Uart_config.DataFormat); // 8 data bit, 1 stop bit, no parity
  SPI_Uart_WriteByte(FCR, 0x06); // reset TXFIFO, reset RXFIFO, non FIFO mode
  SPI_Uart_WriteByte(FCR, 0x01); // enable FIFO mode

  // Perform read/write test to check if UART is working
  SPI_Uart_WriteByte(SPR,'H');
  data = SPI_Uart_ReadByte(SPR);

  if(data == 'H'){ 
    return 1; 
  }
  else{ 
    return 0; 
  }

}

void SPI_Uart_WriteByte(char address, char data)
// Write single byte to register at <address>
{
  long int length;
  char senddata[2];
  senddata[0] = address;
  senddata[1] = data;

  select();
  length = SPI_Write(senddata, 2);
  deselect();
}

long int SPI_Write(char* srcptr, long int length)
// Write string to SC16IS750
{
  for(long int i = 0; i < length; i++)
  {
    spi_transfer(srcptr[i]);
  }
  return length; 
}

void SPI_Uart_WriteArray(char *data, long int NumBytes)
// Write string to THR of SC16IS750
{
  long int length;
  select();
  length = SPI_Write(&TX_Fifo_Address,1);

  while(NumBytes > 16)
  {
    length = SPI_Write(data,16);
    NumBytes -= 16;
    data += 16;
  }
  length = SPI_Write(data,NumBytes);

  deselect();
}

char SPI_Uart_ReadByte(char address)
// Read SC16IS750 register at <address>
{
  char data;

  address = (address | 0x80);

  select();
  spi_transfer(address);
  data = spi_transfer(0xFF);
  deselect();
  return data;  
}

char autoconnect(void)
// Automatically connect to WiFi network using parameters in global variables
{
  // Exit command mode if we haven't already
  SPI_Uart_println("");  
  SPI_Uart_println("exit");
  delay(500);

  // Enter command mode
  SPI_Uart_print("$$$");
  delay(500);

  // Reboot to get device into known state
  Serial.println("Rebooting");
  SPI_Uart_println("reboot");
  delay(3000);

  // Enter command mode
  Serial.println("Entering command mode.");
  SPI_Uart_print("$$$");
  delay(500);

  // Set the security authorization level of the WiFly using <auth_level>
  SPI_Uart_print("set w a ");
  SPI_Uart_println(auth_level); 
  delay(500);
  Serial.print("Set wlan to authorization level ");
  Serial.println(auth_level);

  // Set the WiFly authorization phrase using <auth_phrase>
  SPI_Uart_print("set w p ");
  SPI_Uart_println(auth_phrase);
  delay(500);
  Serial.print("Set security phrase to ");
  Serial.println(auth_phrase);

  // Set the WiFly localport using <port_listen>
  SPI_Uart_print("set i l ");
  SPI_Uart_println(port_listen);
  delay(500);
  Serial.print("Set IP localport to ");
  Serial.println(port_listen);

  // Turn off WiFly UART connection indicators
  SPI_Uart_println("set comm remote 0");
  delay(500);
  SPI_Uart_println("set comm open 0");
  delay(500);
  SPI_Uart_println("set comm close 0");
  delay(500);

  // Join WiFi network <ssid>
  Serial.print("Joining '");
  Serial.print(ssid);
  Serial.println("'");
  delay(100);
  Flush_RX();
  delay(100);
  SPI_Uart_print("join ");
  SPI_Uart_println(ssid);  
  for(int p = 0; p < 5; p++) // Give WiFly time to associate
  {
    Serial.print(".");
    speakjet.print(ping);
    delay(ASSOCIATE_TIMEOUT/5);
  }
  Serial.println("");

  // Flush characters from SC16IS750
  Flush_RX();

  // Check for association
  SPI_Uart_println("show c");
  if(Wait_On_Response_Char(13) != '0')
    // Failed, try again
  {
    Serial.print("Failed to associate with '");
    Serial.print(ssid);
    Serial.println("'\n\rRetrying...");
    Flush_RX();
    autoconnect();
  }
  else
    // Success, SpeakJet says "Associated" and IP address
  {
    Serial.println("Associated!");
    speakjet.print(message);
    delay(1000);
    SPI_Uart_println("get ip");
    get_ip();
  }
}

void Flush_RX(void)
// Flush characters from the SC16IS750 so we only see what we want to see on the terminal
{
  int j = 0;
  while(j < 4000)
  {
    if((SPI_Uart_ReadByte(LSR) & 0x01))
    {
      incoming_data = SPI_Uart_ReadByte(RHR);
    }  
    else
    {
      j++;
    }
  }
}

void Parse_Request(void)
// Parse POST data into separate words, look up, and "say" words using SpeakJet
{
  int j = 0, i = 0, k = 0;
  char a = 0;
  char printflag = 0;
  char message[128];
  char poem_word[16] = { 
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0     };
  char buffer[16] = { 
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0     };
  char sentinel = 0;
  while(j < 4000) // Read in full message
  {
    if((SPI_Uart_ReadByte(LSR) & 0x01))
    {
      incoming_data = SPI_Uart_ReadByte(RHR);
      if(printflag){ 
        message[i-3] = incoming_data;
        i++;    
      }
      else if(incoming_data == '%'){ 
        printflag = 1; 
      }
    }  
    else
    {
      j++;
    }
  }
  i -= 3;
  j = 0;
  while(k<i) // Split <message> into words and say
  {
    while(message[k] != '+' && k < i)
    {
      poem_word[j] = message[k];
      k++;
      j++;
    }
    k++;
    for(int index = 0; index < WORD_LIST_LENGTH; index++)
    {
      if(strcmp(poem_word,SpeakJetLookup[index]) == 0) // Look up word in SpeakJetLookup
      {
        // Serial.print(index,DEC); // Print index of word for debugging
        strcpy_P(buffer, (char*)pgm_read_word(&(SpeakJetDictionary[index]))); // Copy word from program memory using lookup index
        speakjet.print(buffer); // Send string to the VoiceBox
        is_get = 0; // Flag as POST request
        delay(600); // Wait in between words
      }
    }
    j = 0;
    for(char c = 0; c < 16; c++){  // Clear buffers 
      poem_word[c] = 0; 
      buffer[c] = 0; 
    }
  }
}

char Wait_On_Response_Char(char num)
// Check for character number <num> from a response string
{
  i = 1;
  while(1)
  {
    if((SPI_Uart_ReadByte(LSR) & 0x01))
    {
      incoming_data = SPI_Uart_ReadByte(RHR);
      //Serial.print(incoming_data, BYTE); // Print data for debugging
      if(i == num){ 
        return incoming_data; 
      }
      else{ 
        i++; 
      }
    }  
  }
}

void SPI_Uart_println(char *data)
// Write <data> to THR of SC16IS750 followed by a carriage return
{
  SPI_Uart_WriteArray(data,strlen(data));
  SPI_Uart_WriteByte(THR, 0x0d);
}

void HTML_print(char *data)
// Write <data> to THR of SC16IS750 followed by a delay
{
  SPI_Uart_WriteArray(data,strlen(data));
  delay(30);
}

void SPI_Uart_print(char *data)
// Obfuscation of SPI_Uart_WriteArray that uses strlen instead of hardcoded length
{
  SPI_Uart_WriteArray(data,strlen(data));
}

char spi_transfer(volatile char data)
{
  SPDR = data;                    // Start the transmission
  while (!(SPSR & (1<<SPIF)))     // Wait for the end of the transmission
  {
  };
  return SPDR;                    // return the received byte
}

void get_ip(void)
// Parse out IP address of WiFly and "say" it with the speakjet
{
  char returns = 0;
  char ip_index = 0;
  char a = 0;
  // Parse
  while(returns < 6)
  {
    if((SPI_Uart_ReadByte(LSR) & 0x01))
    {
      incoming_data = SPI_Uart_ReadByte(RHR);
      if(incoming_data == 0x0d)
      {
        returns++;
      }
      if(returns > 4)
      {
        //Serial.print(incoming_data,BYTE);
        ip_address[ip_index] = incoming_data;
        ip_index++;
      }
    }
  }
  delay(1000);

  // Say IP address
  while(ip_address[a] != ':')
  {
    if(a < 5){
    }
    else if(ip_address[a] == '.'){ 
      Serial.print(ip_address[a]); 
      speakjet.print(dot); 
      delay(500); 
    }
    else{ 
      Serial.print(ip_address[a]); 
      speakjet.print(numbers[(ip_address[a] - 48)]); 
      delay(500); 
    }
    a++;
  }
  Serial.println("");
  Flush_RX(); // Do not print characters
}


